if( !window.JSAX){
	alert( "JSAX.CJSED is not able to work without JavaScript Abstractions for X(HT)ML (objects/JSAX.js).");
}

require( "proto/String", "proto/Array", "Reference", "Cascade", "Event");

/**
* JSAX.CJSED - Cascading JavaScript Event Declarations
* 
* JavaScript Event Declarations in Cascading -style allmost like CSS.
* @example:
* 
* Copyright (C) 2006 Florian Sax <flosax(at)users.sourceforge.net>
* JSAX.CJSED is protected by the GPL <http://www.gnu.org/copyleft/gpl.html>
*
* @example var file= "DOMinfo.css"; require("CJSED"); JSAX.CJSED.load(file); JSAX.CJSED.cache[file]
*/
JSAX.CJSED = {

	Version: 20060127,

/**
* JSAX.CJSED.load - Loads and activates the CJSED rules of the specified file
*
* @param String file - The path of the cjsed file you want to load
*
* @return Number - The number of selector statements which were successfully added.
*/
	load: function( file){
		JSAX.CJSED.cache[file] = read( file);
		return JSAX.CJSED.publish( JSAX.CJSED.cache[file]);
//		JSAX.Event.start();
	},

/**
* JSAX.CJSED.publish - Load a CJSED string
*
* @param String CJSED_string - The rule(s) you want to load
*
* @return Number - The number of selector statements which were successfully added.
*/
	publish: function( CJSED_string){
		var rules = JSAX.Cascade.splitRules( CJSED_string);
		if( !rules.length){
			return false;
		}
		var added = 0;
		for( var r=0; r< rules.length; r++){
			rules[r].ruleFunction = JSAX.CJSED.makeFuncFromRuleArgs( rules[r].arguments);
			for( var s=0; s < rules[r].selectors.length; s++){
//				JSAX.CJSED.addFuncToList( rules[r].ruleFunction, rules[r].selectors[s]);
				JSAX.CJSED.addToSelector( rules[r].selectors[s], rules[r].ruleFunction);
			}
			if( typeof( rules[r].ruleFunction) == "function"){
				added++;
			}
		}
		JSAX.CJSED.onLoad();
		return added;
	},


/**
* 
*/
	makeFuncFromRuleArgs: function( args){
		var args = args;
		var funcTxt = "";
		for( var i=0; i < args.length; i++){
//			funcTxt += "\n"+JSAX.CJSED.correctFuncArg( args[i][0]+"."+args[i][1]);
			var arg = args[i][0].toString();
			if( !arg.length){
				continue;
			}
			var val = false;
			if( typeof( args[i][1]) != "undefined"){
				var val = args[i][1].toString();
			}
			var styleArg = JSAX.CJSED.styleArg( arg);
			if( styleArg){
				funcTxt += styleArg+" = '"+val+"';\n";
			}else{
				if( arg.pos( ".") == -1
				 && arg.pos( "[") == -1
				 && arg.pos( "(") == -1
				 && arg.pos( ")") == -1
				 && arg.pos( "{") == -1
				 && arg.pos( "}") == -1
				 && arg.pos( "else") != 0
				 && arg.pos( "var") != 0
				 && arg.pos( "typeof") != 0
				 && arg.pos( "new") != 0
				 && arg.pos( "return") != 0){
					arg = "this."+arg;
				}
				if( val){
					arg += " = "+val;
				}
				funcTxt += arg+";\n"
			};
		}
		funcTxt = funcTxt.str_replace( "this.", "elem.");
		funcTxt = funcTxt.str_replace( "this;", "elem;");
		funcTxt = funcTxt.str_replace( "this)", "elem)");
		funcTxt = funcTxt.str_replace( "this ", "elem ");
		funcTxt = funcTxt.str_replace( "this=", "elem=");
		try{
			var func = new Function( "elem, event", funcTxt);
		}catch( e){
			func = "could not crate function: "+e.message+": "+funcTxt;
			alert( func);
		}
		return func;
	},

/**
* 
*/
	styleArg: function( arg){
		var arg = arg.toLowerCase();
		if( !JSAX.reference.CSS.properties.contains( arg)){
			return false;
		}
		if( arg.pos( "-") != -1){
			arg = arg.substring( 0, arg.pos( "-"))
			     +arg.substr( arg.pos( "-")+1 ,1).toUpperCase()
			     +arg.substring( arg.pos( "-")+2);
		}
		arg = "this.style."+arg;
		return arg;
	},

/**
* 
*/
	addToSelector: function( selector, func){
		var DOMpath = "JSAX.CJSED.selectors";

		for( var i=selector.length-1; i >= 0; i--){
			selector[i] = selector[i].stri_replace( ":on", ":");
			DOMpath += "['"+selector[i]+"']";
			var obj = eval( DOMpath);
			if( !obj){
//				alert( DOMpath);
				eval( DOMpath+" = new Array();");
                        }
		}
		var obj = eval( DOMpath);
		obj.push( func);
	},

/**
* 
*/
	checkElem: function( elem, event){
		if( event){
			var eventName = event.type;
		}else{
			var eventName = "";
		}
		var possibleSelectors = new Array( elem.nodeName.toLowerCase());
		if( elem.className){
			possibleSelectors.push( "."+elem.className);
			possibleSelectors.push( elem.nodeName.toLowerCase()+"."+elem.className);
		}
		if( elem.id){
			possibleSelectors.push( "#"+elem.id);
			possibleSelectors.push( elem.nodeName.toLowerCase()+"#"+elem.id);
		}
		var psels = possibleSelectors.length;
		for( var i=0; i < psels; i++){
			possibleSelectors.push( possibleSelectors[i]+":"+eventName);
		}
		possibleSelectors.push( ":"+eventName);
		for( var i=0; i < possibleSelectors.length; i++){
			if( JSAX.CJSED.selectors[possibleSelectors[i]]){
				JSAX.CJSED.runSelectorStatement( JSAX.CJSED.selectors[possibleSelectors[i]], elem, event);
			}
		}
		if( !arguments[2]
		 && elem.parentNode
		 && elem.parentNode.style){
			JSAX.CJSED.checkElem( elem.parentNode, event);
		}
		window.possibleSelectors = possibleSelectors;
	},

/**
* 
*/
	runSelectorStatement: function( selector, elem, event){
		for( var i=0; i < selector.length; i++){
			if( typeof( selector[i]) == "function"){
				try{
					selector[i]( elem, event);
				}catch( e){
					alert( e.message);
				}
			}
		}
		
	},

/**
* 
*/
	clearAllSelectorStatement: function(){
		JSAX.CJSED.selectors = new Array();
	},

/**
* 
*/
	clearSelectorStatement: function( selector){
		if( selector == "*"){
			return JSAX.CJSED.clearAllSelectorStatement();
		}
		JSAX.CJSED.selectors[possibleSelectors[i]] = null;
	},

/**
* 
*/
	onLoad: function(){
		var allElems = document.getElementsByTagName( "*");
		var event = {
			type: "load",
			target: false
		}
		for( var i = 0; i < allElems.length; i++){
			event.target = allElems[i];
			JSAX.CJSED.checkElem( allElems[i], event, "dontFollow");
		}
	},

/**
* 
*/
	commitHeaderTags: function(){
		var tags = document.getElementsByTagName( "script");
		for( var i=0; i < tags.length; i++){
			if( tags[i].type.toLowerCase() == "text/cjsed"){
				if( tags[i].src){
					JSAX.CJSED.load( tags[i].src);
				}
				JSAX.CJSED.publish( tags[i].innerHTML.unescapeHTML());
			}
		}
		var tags = document.getElementsByTagName( "style");
		for( var i=0; i < tags.length; i++){
			if( tags[i].type.toLowerCase() == "text/cjsed"){
				JSAX.CJSED.publish( tags[i].innerHTML.unescapeHTML());
			}
		}
		var tags = document.getElementsByTagName( "link");
		for( var i=0; i < tags.length; i++){
			if( tags[i].type.toLowerCase() == "text/cjsed"){
				if( tags[i].rel){
					JSAX.CJSED.load( tags[i].rel);
				}else if( tags[i].src){
					JSAX.CJSED.load( tags[i].src);
				}else if( tags[i].href){
					JSAX.CJSED.load( tags[i].href);
				}
			}
		}
		JSAX.CJSED.onLoad();
	},
	cache: new Array(),
	selectors: new Array()
};

window.onload = JSAX.CJSED.commitHeaderTags;
window.importCJSED = JSAX.CJSED.load;











