if( !window.JSAX){
	alert( "JSAX.CSV is not able to work without JavaScript Abstractions for X(HT)ML (objects/JSAX.js).");
}
require( "proto/String");

/**
* JSAX.CSV
* This library handles Comma Seperated Value strings
*
* test with:
require( "CSV"); window.adr = JSAX.CSV.load( "../../content/adr.csv", ";");adr

window.test = JSAX.CSV.implode( window.adr, ";;");test

* Copyright (C) 2005 Florian Sax <flosax(at)users.sourceforge.net>
* JSAX.CSV is under the terms of the GNU General Public License <http://www.gnu.org/copyleft/gpl.html>
* @parent JSAX.CSV
*/
JSAX.CSV = {
	Version: 20060210,

// @param generalFieldSeparators - Defines some field separation string or charcter

	generalFieldSeparators: [';;', ',,', ';', ',', '\t'],

// @param lineSeparator - Defines the field separation string or charcter

	lineSeparator: '\n',

/**
* @desc Returns the date of a CSV file as an array
*
* @param String fileName   - The URL of a text file ( must be on the same host this script is on)
* @param String [fieldSep] - Defines the field separation string or charcter
* @param String [lineSep]  - Defines the line separation string or character
*
* @return Array - Data of CSV file
*
* @example require( "CSV"); JSAX.CSV.load( "test.csv")
* @example require( "CSV"); JSAX.CSV.load( "test.csv", ",", "\n")
*/
	load: function( fileName, fieldSep, lineSep){
		var fileContent = read( fileName);
		return JSAX.CSV.explode( fileContent, arguments[1], arguments[2]);
	},

/**
* @desc Detects the field separator. This function will be used by load() and explode() if you don't specify the field separator
*
* @param String CSVstring - The CSV string
*
* @return String - The most possible field separator
*/
	detectFieldSeparator: function( CSVstring){
		var lines = CSVstring.split( JSAX.CSV.lineSeparator);
		var fn = 0;
		var sep = ",";
		for( var s = 0; s < JSAX.CSV.generalFieldSeparators.length; s++){
			var psep = JSAX.CSV.generalFieldSeparators[s];
			var fn1 = lines[0].indexCount( psep);
			var fn2 = lines[1].indexCount( psep);
			if( fn1 > fn && fn1 == fn2){
				sep = psep;
				fn = fn1;
			}
//			alert(psep+" "+fn1+" "+fn2+" ="+sep);
		}
		return sep;
	},

/**
* @desc Returns the date of a CSV string as an array
*
* @param String CSVstring  - A CSV string
* @param String [fieldSep] - Defines the field separation string or charcter
* @param String [lineSep]  - Defines the line separation string or character
*
* @return Array - Data of CSV string
*/
	explode: function( CSVstring, fieldSep, lineSep){
		if( arguments[1]){
			var sepF = arguments[1];
		}else{
			var sepF = JSAX.CSV.detectFieldSeparator( CSVstring);
		}
		if( arguments[2]){
			var sepL = arguments[2];
		}else{
			var sepL = JSAX.CSV.lineSeparator;
		}

		var lines = CSVstring.trimSplit( sepL);
		for( var l = 0; l < lines.length; l++){
			lines[l] = lines[l].chopEnd( sepF).trimSplit( sepF);
		}
		return lines;
	},

/**
* @desc Makes a CSV string of an array
*
* @param Array - Array of data wich will be used to make the CSV string
* @param String [fieldSep] - Defines the field separation string or charcter
* @param String [lineSep]  - Defines the line separation string or character
*
* @return string - CSV data of the given array
*
* @example require( "CSV"); JSAX.CSV.implode( JSAX.CSV.load( "test.csv"), ";;", "\n")
*/
	implode: function( array, fieldSep, lineSep){
		if( arguments[1]){
			var sepF = arguments[1];
		}else{
			var sepF = JSAX.CSV.generalFieldSeparators[0];
		}
		if( arguments[2]){
			var sepL = arguments[2];
		}else{
			var sepL = JSAX.CSV.lineSeparator;
		}

		lines = new Array();
		for( var l = 0; l < array.length; l++){
			lines[l] = array[l].join( sepF);
		}
		var CSVstring = lines.join( sepL);
		return CSVstring;
	}
};


