/**
* JSAX.Cookie
* written by Florian Sax 20060128
*
* Copyright (C) 2006 Florian Sax <flosax(at)users.sourceforge.net>
* JSAX.Cookie is under the terms of the GNU General Public License <http://www.gnu.org/copyleft/gpl.html>
*/

if( !window.JSAX){
	alert( "JSAX.Cookie is not able to work without JavaScript Abstractions for X(HT)ML (objects/JSAX.js).");
}


require( "proto/String");
JSAX.Cookie = {
	Version: 20060204,


	data: {},
/**
* JSAX.Cookie.set Sets a Cookie with the given name and value and returns the new cookie array
*
* @param string name       - Name of the cookie
* @param string value      - Value of the cookie
* @param string [expires]  - Expiration date of the cookie (default: end of current session)
* @param string [path]     - Path where the cookie is valid (default: path of calling document)
* @param string [domain]   - Domain where the cookie is valid (default: domain of calling document)
* @param string [secure]   - Boolean value indicating if the cookie transmission requires a secure transmission
*
* @return array Updated cookies
*/
	set: function( name, value, expires, path, domain, secure){
		document.cookie = name+"="+escape( value)+
			( ( expires)	? "; expires="+expires.toGMTString()	: "")+
			( ( path)	? "; path="+path			: "")+
			( ( domain)	? "; domain="+domain			: "")+
			( ( secure)	? "; secure"				: "");
		return JSAX.Cookie.read();
	},

/**
* JSAX.Cookie.drop Deletes the specified cookie and returns the new cookie array
*
* @param string name     - Name of the cookie
* @param string [path]   - Path of the cookie (must be same as path used to create cookie)
* @param string [domain] - Domain of the cookie (must be same as domain used to create cookie)
*
* @return array Updated cookies
*/
	drop: function( name, path, domain){
		if( JSAX.Cookie.has( name)){
        		document.cookie = name+"="+
				( ( path)	? "; path="+path	: "")+
				( ( domain)	? "; domain="+domain	: "")+
				"; expires=Thu, 02-Jan-70 00:00:01 GMT";
		}
		return JSAX.Cookie.read();
	},

/**
* JSAX.Cookie.has Returns true if the cookie exists
*
* @param string name - The name of the cookie
*
* @return boolean True or false
*/
	has: function( name){
		JSAX.Cookie.read();
		if( typeof( JSAX.Cookie.data[name]) != "undefined"){
			return true;
		}
		return false;
	},

/**
* JSAX.Cookie.get Returns the value of a cookie or false if it does not exists
*
* @param string name - Name of the cookie
*
* @return string Cookie value or boolean false
*/
	get: function( name){
		JSAX.Cookie.read();
		if( typeof( JSAX.Cookie.data[name]) != "undefined"){
			return JSAX.Cookie.data[name]
		}
		return false;
	},

/**
* JSAX.Cookie.clear Deletes all cookies and returns the new cookie array
*
* @return array Updated cookies
*/
	clear: function(){
		JSAX.Cookie.read();
		for( var atr in JSAX.Cookie.data){
			JSAX.Cookie.drop( atr);
		}
		return JSAX.Cookie.read();
	},

/**
* JSAX.Cookie.read Reads the cookies in the cache array and returns it
*
* @return array Cookies
*/
	read: function(){
		var vars = document.cookie.trimSplit( ";");
		var obj = {};
		for( var i = 0; i < vars.length; i++){
			var kv = vars[i].trimSplit( "=");
			if( typeof( kv[1]) == "string"){
				obj[kv[0]] = unescape( kv[1]);
			}else{
				obj[kv[0]] = true;
			}
		}
		return JSAX.Cookie.data = obj;
	}
};

