/**
* JSAX.JSDoc
* JavaScript Documentation generation similar to JavaDoc
* written by Florian Sax 20060128
*
* test with:
require("comming/JSDoc"); JSAX.JSDoc.read("objects/proto/String.js")

* Copyright (C) 2006 Florian Sax <flosax(at)users.sourceforge.net>
* JSAX.JSDoc is under the terms of the GNU General Public License <http://www.gnu.org/copyleft/gpl.html>
*/

 
if( !window.JSAX){
	alert( "JSAX.JSDoc is not able to work without JavaScript Abstractions for X(HT)ML (objects/JSAX.js).");
}

require( "proto/String", "proto/Array");

require( "GUI/DOMinfo/DOMinfo");

JSAX.JSDoc = {
	Version: 20060212,

	docData: [],
	docElem: [],

	types: ["function", "object", "string", "number", "boolean", "array", "element", "mixed", "integer", "decimal", "binary"],
	typeCorrections: [ // not implemented so far
		["bin", "binary"],
		["int", "integer"],
		["str", "string"],
		["bool", "boolean"],
		["dec", "decimal"],
		["num", "number"],

		["binary", "binary string"],
		["decimal", "decimal number"]],

	object: {
		name: "",
		type: "",
		desc: "",
		code: "",

		example: [],

		argNames: [],
		args: [],
		params: [],
		"return": "",

		children: []
	},

	docStart: "/**",

	makeHTML: function( obj){
		if( obj.ignore){
			return "";
		}
		if( obj.type == "function"){
			if( obj["args"]){
				var funcArgs = [];
				for( var i = 0; i < obj["args"].length; i++){
					funcArgs.push( "<i>"+obj["args"][i]["type"]+"</i> "+obj["args"][i]["name"]+
						( obj["args"][i]["desc"] ? " - "+obj["args"][i]["desc"] : ""));
				}
				if( funcArgs.length){
					funcArgs = "<b>Arguments:</b>"+funcArgs.inbedd( "<li>", "</li>")+"<br/>";
				}else{
					funcArgs = "";
				}
			}

			var funcTxtArgs = ( obj["args"] && obj["args"].length ? " "+obj["args"].doEach( function(){ return "<i>"+this["type"]+"</i> "+this["name"]; }).join( ", ") : "");
			var funcTxt = ( obj["return"] ? "<i>"+obj["return"]["type"]+"</i> " : "")+
				obj["name"]+"("+funcTxtArgs+")";
		}
		return "<h2><i>"+obj["type"]+"</i> "+obj["name"]+"</h2>\n"+

			( ( obj["version"]) ? "<b>Version:</b> "+obj["version"]+"<br/><br/>" : "")+
			( ( funcTxt || obj.desc) ? "<b>Description:</b><br/>" : "")+
				( funcTxt ? funcTxt+"<br/><br/>\n" : "")+
				( obj.desc ? obj.desc+"<br/><br/>\n" : "")+

			( ( funcArgs && funcArgs.length) ? funcArgs : "")+

			( obj["return"] ? "<b>Return:</b> <i>"+
				obj["return"]["type"]+"</i>"+
				( ( obj["return"]["name"] || obj["return"]["desc"]) ? " - "+obj["return"]["name"]+
					" "+obj["return"]["desc"] : "")+"<br/><br/>" : "")+

			( ( obj.childs && obj.childs.length) ? "<b>Clildren:</b>"+
				 obj.childs.inbedd( "<li>", "</li>")+"<br/>" : "")+

			( ( obj["example"] && obj["example"].length) ? "<b>Examples:</b><br/><code>"+
				obj["example"].inbedd( "<div class='xampl' onclick='JSAX.DOMinfo.activateElement( this);'>", "</div>")+"</code><br/>" : "")+

			( obj.code ? "<b>Code:</b><br/><code>"+obj.code+"</code><br/>\n" : "\n");
	},

	openExample: function( elem){
		var call = elem.innerHTML.unescapeHTML().trim();
		var elem = JSAX.DOMinfo.makeElem( call);
		JSAX.WFM.open( elem, 'example');
	},




/**
* @desc JSAX.JSDoc.read Returns an array of the documentation data gathered from the file
*
* @param string fileName - URL to the file wich will be loaded ( only from the host where this script is on)
*
* @return array Documentation
*/
	read: function( fileName){
		var content = read( fileName);
		if( !JSAX.JSDoc.docData[fileName]){
			JSAX.JSDoc.docData[fileName] = JSAX.JSDoc.tokenIze( content);
			JSAX.JSDoc.docElem[fileName] = JSAX.JSDoc.make( JSAX.JSDoc.docData[fileName]);
			JSAX.JSDoc.docElem[fileName].setAttribute( "name", fileName);
		}

		return JSAX.JSDoc.docElem[fileName];
	},

/**
* @desc JSAX.JSDoc.make Returns an array of the documentation data gathered from the jsContent string
*
* @param string jsContent - The JavaScript wich will be used to generate the Documentation
*
* @return array Documentfunctionation
*/
	make: function( docuTokens){
		if( typeof( docuTokens) != "object"){
			return false;
		}
		var docu = document.createElement( "div");
		docu.className = "JSDoc";

		for( var i = 0; i < docuTokens.length; i++){
			if( typeof( docuTokens[i]) != "object"){
				continue;
			}
			var ob = docuTokens[i];
			var docuElem = document.createElement( "div");
			docuElem.setAttribute( "class", "JSDoc");
			docuElem.setAttribute( "name", ob["name"]);
			docuElem.innerHTML = JSAX.JSDoc.makeHTML( ob);
			docu.appendChild( docuElem);
		}

		return docu;
	},

	tokenIzeNew: function( jsContent){
		var r = jsContent;
		var doc = [];
		while( r.pos( "/**") != -1){
			pds = r.pos( "/**");
			pde = r.substr( pds).pos( "*/") +pds +2;
			pce = r.substr( pde).pos( ";") +1 +pde;
			var part = r.substr( pds, pce);
			alert( part);
			doc.push( JSAX.JSDoc.tokenIzePart( part));
			r = r.substr( pce);
		}
		return doc;
	},
	tokenIze: function( jsContent){
		var lines = jsContent.split( "\n");
		var part = "";
		var docs = [];
		var parent = "window";
		for( var i = 0; i < lines.length; i++){
			part += "\n"+lines[i];
			if( JSAX.JSDoc.isInteresting( lines[i])){
				var docPart = JSAX.JSDoc.tokenIzePart( part);
				part = "";

				if( docPart["parent"]){
					parent = docPart["parent"].chopEnd( ".");
				}else if( docPart["type"] == "object"){
					if( docPart["name"].pos( ".") != -1){
						parent = docPart["name"];
					}
				}
				if( parent && docPart["name"].pos( ".") == -1 && parent != docPart["name"]){
					var pa = parent.split( ".");
					for( var p = pa.length-1; p >= 0; p--){
						docPart["name"] = docPart["name"].chopBegin( pa[p]+".");
					}
					docPart["name"] = parent+"."+docPart["name"];
				}
				docPart["name"] = docPart["name"].chopBegin( "window.");
				docPart["parent"] = parent;

				docs.push( docPart);
			}
		}

		return docs;
	},

	isInteresting: function( codePart){
		var line = codePart.trim();
		bl = ["||", "&&", "==", "!=", ">=", "<="];
		for( var i = 0; i < bl.length; i++){
			if( line.pos( bl[i]) != -1){
				return false;
			}
		}
		var i = 0;
		while( line.whitePos() != -1){
			if( i < 3){
				line = line.substr( 0, line.whitePos())+
					line.substr( line.whitePos()+1).trim();
				i++;
			}else{
				line = line.substr( 0, line.whitePos());
			}
		}

		if( ( ( line.pos( "function") == 0
		 || line.pos( "function(") != -1
		 || line.pos( "={") != -1
		 || line.pos( ":{") != -1)
		 && line.pos( "}") == -1)
		 || line.pos( "JSDoc here") != -1){
			return true;
		}
		return false;

		// other tech
		s = [	["//",          0.9],
			["*/",          0.9],
			["/*",          0.9],
			["(",           0.2],
			[")",           0.2],
			["{",           0.5],
			["=",           0.5],
			["function",    0.1],
			["for(",         -2],
			["if(",          -2],
			["while(",       -2],
			["do(",          -2],
			[";",          -0.5],
			["}",          -0.5]];
		var p = 0;
		for( var i = 0; i < s.length; i++){
			if( codePart.pos( s[i][0]) != -1){
				p += s[i][1];
			}
		}
		if( p >= 1){
			return true;
		}
		return false;
	},

	tokenIzePart: function( jsContentPart){

		var line = jsContentPart.substr( jsContentPart.lpos( "\n"));

		var keyStartPos = 0;
		var keyEndPos = 0;
		if( line.pos( ":") > 0){
			keyEndPos = line.pos( ":");
		}
		if( line.pos( "=") > 0){
			keyEndPos = line.pos( "=");
		}
		if( keyEndPos < 1){
			keyEndPos = line.length -1;
		}
		var key = line.substr( 0, keyEndPos).trim();
		if( key.whitePos() != -1){
			key = key.chopBegin( "function", "var").trim();
		}

		var docPart = [];
		docPart["name"] = key;

		if( line.pos( "function") != -1
		 && line.pos( "(")){
			var args = line.substring( line.pos( "(")+1, line.pos( ")")).trimSplit( ",").removeEmpty();
			docPart["type"]	= "function";
			docPart["argNames"] = args;
			docPart["code"] = "";
		}else{
			docPart["type"]	= "object";
		}
		
		var funcArgs = 0;

		var docStartPos = jsContentPart.pos( JSAX.JSDoc.docStart);
		var endPos = false;
		if( docStartPos != -1){
			docStartPos += JSAX.JSDoc.docStart.length -1;
			docEndPos = jsContentPart.substr( docStartPos).pos("*/");

			var doc = jsContentPart.substr( docStartPos, docEndPos);

//			alert( jsContentPart+"\n=>"+docStartPos+"\n=>"+docEndPos+"\n="+doc);
			docPart = docPart.fusionMe( JSAX.JSDoc.readCodeDoc( doc));
		}

		if( docPart["argNames"] && ( !docPart["args"] || docPart["argNames"].length > docPart["args"].length)){
//			alert( docPart["argNames"].length+" "+docPart["args"].length);
			if( !docPart["args"]){
				docPart["args"] = [];
			}

			for( var i = 0; i < docPart["argNames"].length; i++){
				if( docPart["args"][i]){
					var arg = docPart["args"][i];
				}else{
					var arg = [];
				}
				docPart["args"][i] = {
					"name": ( arg["name"] ? arg["name"] : docPart["argNames"][i].toString()),
					"type": ( arg["type"] ? arg["type"] : ""),
					"desc": ( arg["desc"] ? arg["desc"] : "")
				};
			}

		}
		return docPart;
	},

	solvArgString: function( entry){
		entry = entry.trim();
		var seps = [ " ", "\t"];
		var e = entry.smartSplit( seps, 1, 1);
		e = e.remove( "-");
//		alert( e);
		var a = [];
		a["name"] = "";
		a["desc"] = "";
		var w = 0;
		var d = false;
		if( e[1] && JSAX.JSDoc.types.indexOf( e[1].toLowerCase()) != -1){
			a["type"] = e[1];
			a["name"] = e[0];
			if( e.length == 2){
				d = true;
			}else{
				w = 1;
			}
		}else if( JSAX.JSDoc.types.indexOf( e[0].toLowerCase()) != -1){
			a["type"] = e[0];
			a["name"] = e[1];
			if( e.length < 3){
				d = true;
			}else{
				w = 1;
			}
		}else{
			a["type"] = "unknown";
			if( e.length == 1){
				a["name"] = e[0];
				d = true;
			}
		}
		if( !d){
			a["desc"] = entry.substr( entry.pos( e[w]) +e[w].length).trim().chopBegin( ["-", ":", "="]).trim();
		}
		return a;
	},

	readCodeDoc: function( doc){
		var docEntries = doc.trimSplit( "*");
		var doc = [];
		var args = 0;
		for( var i = 0; i < docEntries.length; i++){
			var entry = docEntries[i];
			var typ = "desc";
			if( entry.pos( "@") == 0){
				typ = entry.substring( 1, entry.whitePos()).toLowerCase();
				entry = entry.substr( entry.whitePos()).trim();
				typ = typ.chopEnd( [":", "=", "-"]).trim();
				entry = entry.chopBegin( [":", "=", "-"]).trim();
			}

			if( typ == "param"){
				if( !doc["params"])doc["params"] = [];
				doc["params"].push( entry);
				if( !doc["args"])doc["args"] = [];
				doc["args"].push( JSAX.JSDoc.solvArgString( entry));
			}else if( typ == "return"){
				doc["return"] = JSAX.JSDoc.solvArgString( entry);
			}else if( typeof( JSAX.JSDoc.object[typ]) != "undefined"
			       && typeof( JSAX.JSDoc.object[typ].push) == "function"){
				if( !doc[typ])doc[typ] = [];
				doc[typ].push( entry);
			}else{
				if( typeof( doc[typ]) != "string"){
					doc[typ] = "";
				}
				if( doc[typ].length){
					doc[typ] += "\n";
				}
				doc[typ] += entry;
			}
//			docPart.push( JSAX.JSDoc.makeEntry( docEntries[i]));
		}
		if( typeof( doc["desc"]) == "string"){
			doc["desc"] = doc["desc"].trim().txt2html();
		}
		if( typeof( doc["example"]) == "object"){
			doc["example"] = doc["example"].doEach( "txt2html");
		}

		return doc;
	}

};

