/**
* JSAX.code
* 
* written by Florian Sax 20060205
*
* Copyright (C) 2006 Florian Sax <flosax(at)users.sourceforge.net>
* JSAX.code is protected by the GPL <http://www.gnu.org/copyleft/gpl.html>
*
* @version 20060217
*
* @example require( "code"); JSAX.code
*/

if( !window.JSAX){
	alert( "JSAX.code is not able to work without JavaScript Abstractions for X(HT)ML (objects/JSAX.js).");
}

JSAX.code = {
	Version: 20060217,
	Doc: 20060217,

	charSets: {
			bin: "01",
			oct: "01234567",
			dec: "0123456789",
			hex: "0123456789ABCDEF",
			ascii: ""
	},

	zeroFill: function( string, len, set){
		var zeroChar = "0";
		if( set){
			zeroChar = set.substr( 0, 1);
		}
		var string = string.toString();
		while( string.length < len){
			string = zeroChar+string;
		}
		return string;
	},

/**
* JSAX.code.fromDec   - Encodes a decimal value with a charSet
*
* @param Number value - The value which will be converted
* @param String set   - The charSet you want ( 'oct', 'hex', 'bin', '01', '1234')[default='hex']
*
* @return String      - The encoded value
*
* @validation "FF" == JSAX.code.fromDec( 255)
* @validation "11111111" == JSAX.code.fromDec( 255, "bin")
*
* @example JSAX.code.fromDec( 255)
* @example JSAX.code.fromDec( 255, "bin")
*/
	fromDec: function( value, set){
		if( !set){
			set = "hex";
		}
		if( JSAX.code.charSets[set.toLowerCase()]){
			var set = JSAX.code.charSets[set.toLowerCase()];
		}

		var value = parseInt( value);
		var mul = set.length;
		var done = false;
		var code = "";
		while( value){
			var rest = value %( mul);
			var pos = rest /( mul /set.length);
			code = set.charAt( pos)+code;
			value -=  rest;
			mul *= set.length;
		}
		return code;
	},

/**
* JSAX.code.toDec     - Decodes a string with a charSet to a decimal value
*
* @param String value - The with set encoded value
* @param String set   - The charSet value is encoded with ( 'oct', 'hex', 'bin', '01', '1234')[default='hex']
*
* @return Number      - The decimal of value or boolean false if the value contains illegal characters.
*
* @validation 255 == JSAX.code.toDec( "FF")
* @validation 7 == JSAX.code.toDec( "111", "bin")
*
* @example JSAX.code.toDec( "FF")
* @example JSAX.code.toDec( "111", "bin")
*/
	toDec: function( value, set){
		if( !set){
			set = "hex";
		}
		if( JSAX.code.charSets[set.toLowerCase()]){
			var set = JSAX.code.charSets[set.toLowerCase()];
		}

		var value = value.toString();

		if( set == set.toUpperCase()){
			value = value.toUpperCase();
		}
		if( set == set.toLowerCase()){
			value = value.toLowerCase();
		}

		var dec = 0;
		var chr = "";
		var val = -1;
		var mul = 0;
		for( var i = value.length; i > 0; i--){
			chr = value.substr( i -1, 1);
			val = set.pos( chr);
			if( val == -1){
				return false;
			}
			if( !mul){
				mul = 1;
			}else{
				mul *= set.length;
				val *= mul;
			}
			dec += val;
		}
		return dec;
	},

/**
* JSAX.code.toDec       - Recodes a string from a charSet to another charSet
*
* @param String value   - The with set encoded value
* @param String fromSet - The charSet value is encoded with ( 'oct', 'hex', 'bin', '01', '1234')[default='hex']
* @param String toSet   - The charSet value should get encodet with [default='hex']
*
* @return Number        - The decimal of value or boolean false if the value contains illegal characters.
*
* @validation 255 == JSAX.code.fromTo( "FF", "hex", "dec")
* @validation "11111111" == JSAX.code.fromTo( "FF", "hex", "bin")
* @validation "FF" == JSAX.code.fromTo( 255, "dec")
* @validation "111" == JSAX.code.fromTo( 7, "dec", "bin")
* @validation 7 == JSAX.code.fromTo( 111, "bin", "dec")
*
* @example JSAX.code.fromTo( "FF", "hex", "dec")
* @example JSAX.code.fromTo( "FF", "hex", "bin")
* @example JSAX.code.fromTo( 255, "dec")
* @example JSAX.code.fromTo( 7, "dec", "bin")
* @example JSAX.code.fromTo( 111, "bin", "dec")
*/
	fromTo: function( value, fromSet, toSet){
		var value = JSAX.code.toDec( value, fromSet);
		value = JSAX.code.fromDec( value, toSet);
		return value;
	},
/**
* binary
* @parent = JSAX.code
*/
	bin: {
/**
* @parent = JSAX.code.bin
*/
		toDec: function( val){
			return JSAX.code.toDec( val, "bin");
		},
		toOct: function( val){
			return JSAX.code.fromTo( val, "bin", "oct");
		},
		toHex: function( val){
			return JSAX.code.fromTo( val, "bin", "hex");
		},

		fromDec: function( val){
			return JSAX.code.fromDec( val, "bin");
		},
		fromOct: function( val){
			return JSAX.code.fromTo( val, "oct", "bin");
		},
		fromHex: function( val){
			return JSAX.code.fromTo( val, "hex", "bin");
		}
	},
/**
* octal
* @parent = JSAX.code
*/
	oct: {
/**
* @parent = JSAX.code.oct
*/
		toDec: function( val){
			return JSAX.code.toDec( val, "oct");
		},
		toBin: function( val){
			return JSAX.code.fromTo( val, "oct", "bin");
		},
		toHex: function( val){
			return JSAX.code.fromTo( val, "oct", "hex");
		},

		fromDec: function( val){
			return JSAX.code.fromDec( val, "oct");
		},
		fromBin: function( val){
			return JSAX.code.fromTo( val, "bin", "oct");
		},
		fromHex: function( val){
			return JSAX.code.fromTo( val, "hex", "oct");
		}
	},

/**
* hexadecimal
* @parent = JSAX.code
*/
	hex: {
/**
* @parent = JSAX.code.hex
*/
		toDec: function( val){
			return JSAX.code.toDec( val, "hex");
		},
		toBin: function( val){
			return JSAX.code.fromTo( val, "hex", "bin");
		},
		toOct: function( val){
			return JSAX.code.fromTo( val, "hex", "oct");
		},

		fromDec: function( val){
			return JSAX.code.fromDec( val, "hex");
		},
		fromBin: function( val){
			return JSAX.code.fromTo( val, "bin", "hex");
		},
		fromOct: function( val){
			return JSAX.code.fromTo( val, "oct", "hex");
		}
	},

/**
* decimal
* @parent = JSAX.code
*/
	dec: {
/**
* @parent = JSAX.code.dec
*/
		toBin: function( val){
			return JSAX.code.fromDec( val, "bin");
		},
		toOct: function( val){
			return JSAX.code.fromDec( val, "oct");
		},
		toHex: function( val){
			return JSAX.code.fromDec( val, "hex");
		},

		fromBin: function( val){
			return JSAX.code.toDec( val, "bin");
		},
		fromOct: function( val){
			return JSAX.code.toDec( val, "oct");
		},
		fromHex: function( val){
			return JSAX.code.toDec( val, "hex");
		}
		
	},

/**
* bin2dec - Binary to deciaml convertion
*
* @parent = JSAX.code
*
* @param Binary val - The binary data you wan to convert to decimal
*
* @return Integer - The decimal number of val
*/
	bin2dec: function( val){
		return JSAX.code.toDec( val, "bin");
	},
	oct2dec: function( val){
		return JSAX.code.toDec( val, "oct");
	},
	hex2dec: function( val){
		return JSAX.code.toDec( val, "hex");
	},

/**
* dec2bin - Decimal to binary convertion
*
* @parent = JSAX.code
*
* @param Binary val - The decimal number you want to convert to binary
*
* @return Binary - The binary of val
*/
	dec2bin: function( val){
		return JSAX.code.fromDec( val, "bin");
	},
	dec2oct: function( val){
		return JSAX.code.fromDec( val, "oct");
	},
	dec2hex: function( val){
		return JSAX.code.fromDec( val, "hex");
	},

// from bin

	bin2oct: function( val){
		return JSAX.code.fromTo( val, "bin", "oct");
	},
	bin2hex: function( val){
		return JSAX.code.fromTo( val, "bin", "hex");
	},


// from oct
	oct2bin: function( val){
		return JSAX.code.fromTo( val, "oct", "bin");
	},
	oct2hex: function( val){
		return JSAX.code.fromTo( val, "oct", "hex");
	},

// from hex
	hex2bin: function( val){
		return JSAX.code.fromTo( val, "hex", "bin");
	},
	hex2oct: function( val){
		return JSAX.code.fromTo( val, "hex", "oct");
	}

};

// add ascii to the charsets
JSAX.code.charSets.ascii = "";
for( var i = 0; i < 256; i++){
	JSAX.code.charSets.ascii += String.fromCharCode( i);
}

