if( !window.JSAX){
	alert( "JSAX.proto.Array needs JavaScript Abstractions for X(HT)ML (objects/JSAX.js) to work.");
}


/**
* JSAX.proto.Array - Array prototype library
* this is a collection of usefull javascript functions
* written by Florian Sax 20051217
*
* Copyright (C) 2006 Florian Sax <flosax(at)users.sourceforge.net>
* JSAX.proto.Array is protected by the GPL <http://www.gnu.org/copyleft/gpl.html>
*
* @parent JSAX
*
* @example require( "proto/Array"); ["hans", "peter", "anton", "beton"]
*/
JSAX.proto.Array = {
	"JSAX.proto.Array Version": 20060211,

/**
* Check if the array contains something
*
* @parent JSAX.proto.Array
*
* @param Mixed find - The object or key which you look for
*
* @return Boolean - True if there is such a key or value in this array
*
* @example ["hans", "peter", "anton", "beton"].contains( "anton")
* @example ["hans", "peter", "anton", "beton"].flip().contains( "anton")
*/
	contains: function( find){
		if( this[find]){
			return true;
		}
		for( var arg in this){
			if( this[arg] == find){
				return true;
			}
		}
		return false;
	},
/**
* The same as indexOf for strings, but for arrays
*
* @param String find - The object wich needs to be found
*
* @return String - The key oft the object in the array or -1 if not found
*
* @example ["hans", "peter", "anton", "beton"].indexOf( "anton")
*/
	indexOf: function( find){
		var k = this.keys();
		for( var i = 0; i < k.length; i++){
			if( this[k[i]] == find){
				return k[i];
			}
		}
		return -1;
	},
/**
* The same as substr, but for array
*
* @param Integer from - The start of the range
* @param Integer [to] - The length of the range from start (default=rest)
*
* @return Array - The part of the array wich is in the defined range
*
* @example ["hans", "peter", "anton", "beton"].subary(1, 2)
*/
	subary: function( from, to){
		if( !from){
			from = 0;
		}else{
			var from = parseInt( from);
		}
		if( !to){
			var to = this.length
		}
		to += from;
		return this.subarray( from, to);
	},
/**
* The same as substring, but for array
*
* @param Integer from - The start of the range
* @param Integer [to] - The last key that will match (default=rest)
*
* @return Array - The part of the array wich is in the defined range
*
* @example ["hans", "peter", "anton", "beton"].subarray(1, 2)
*/
	subarray: function( from, to){
		if( !from){
			from = 0;
		}else{
			var from = parseInt( from);
		}
		if( !to){
			var to = this.length
		}
		var ar = [];
		for( var i = from; i < to; i++){
			ar.push( this[i]);
		}
		return ar;
	},
/**
* Exchanges the keys and values of the array
*
* @return Array - The array with keys and values "fliped"
*
* @example ["hans", "peter"].flip()["peter"]
*/
	flip: function(){
		var obj = [];
		var k = this.keys();
		for( var i = 0; i < k.length; i++){
			obj[this[k[i]]] = k[i];
		}
		return obj;
	},
/**
* Returns the keys of the array ( this function ignores the JSAX.proto.Array objects)
*
* @return Array - The keys
*
* @example ["hans", "peter"].reverse().keys()
*/
	keys: function(){
		var obj = [];
		for( var ky in this){
			if( ky != "JSAX" && typeof( JSAX.proto.Array[ky]) == "undefined"){
				obj.push( ky);
			}
		}
		return obj;
	},
/**
* This is a alias to keys()
*/
	indexes: function(){
		return this.keys();
	},

/**
* Returns the a copy of the array without all the false values
*
* @return Array - The cleaned array 
*
* @example ["hans", "", false, "kunz"].removeEmpty().keys()
*/
	removeEmpty: function(){
		return this.remove( false);
	},
	remove: function( b){
		var k = this.keys();
		var a = [];
		for( var i=0; i < k.length; i++){
			if( this[k[i]] != b){
				a.push( this[k[i]]);
			}
		}
		return a;
	},

/**
* Simular to join(), but for tags or so
*
* @param String before - The string which should be added before every entry
* @param String after - The string which should be added after every entry
*
* @return String - The inbedded array
*
* @example ["hans", "kunz"].inbedd( "<li>", "</li>\n")
* @example ["hans", "kunz"].inbedd( "- ", "<br/>\n")
*/
	inbedd: function( before, after){
		if( !this.length){
			return "";
		}
		return before+this.join( after+before)+after;
	},

/**
* fusionMe Combines array with this. variables that already exist in this array get overwritten.
*
* @param Object array - The array you want to use to update this array
*
* @return Array - The combined array
*
* @example ["hans", "kunz"].removeEmpty().keys()
*/
	fusionMe: function( array){
		if( typeof( array) != "object"){
			return false;
		}
		var k = array.keys();
		var a = this;
		for( var i = 0; i < k.length; i++){
			a[k[i]] = array[k[i]];
		}
		return a;
	},

/**
* fusionIt Combines array with this. variables that already exist in this array overwrite variables in the argument array.
*
* @param Object array - The array you want to combine with this array
*
* @return Array - The combined array
*
* @example ["hans", "kunz"].removeEmpty().keys()
*/
	fusionIt: function( array){
		if( typeof( array) != "object"){
			return false;
		}
		var k = this.keys();
		var a = array;
		for( var i = 0; i < k.length; i++){
			a[k[i]] = this[k[i]];
		}
		return a;
	},

/**
* Returns the number of objects in the array. This function counts also units with non numeric keys. but not JSAX protoype functions.
*
* @return Number - The amount of objects in the array
*
* @example ["hans", "kunz"].count().keys()
*/
	count: function(){
		return this.keys().length;
	},



	allSubArgs: function( arg){
		var k = this.keys();
		if( !k.length || typeof( this[k[0]]) != "object"){
			return false;
		}
		var a = [];
		for( var i = 0; i < k.length; i++){
			if( typeof( this[k[0]]) == "object" && typeof( this[k[0]][arg]) != "undefined"){
				a.push( this[k[0]][arg]);
			}
		}
		return a;
	},

	doEach: function( func, args, args2){
		var k = this.keys();
		if( !k.length){
			return false;
		}
		var args = [];
		for( var i = 1; i < arguments.length; i++){
			args.push( arguments[i]);
		}

		var a = this;
		for( var i = 0; i < k.length; i++){
			a[k[i]] = args.asArcsInFunc( func, this[k[i]]);
		}
		return a;
	},

	asArcsInFunc: function( func, target){

		if( typeof( target) != "undefined"){
			if( typeof( func) == "function"){
				target["tempFunction"] = func;
				var func = "tempFunction";
			}
		}else{
			if( typeof( func) != "function"){
				var target = window;
			}else{
				var target = {};
				target["tempFunction"] = func;
				var func = "tempFunction";
			}
		}
		if( typeof( target[func]) != "function"){
			return false;
		}

		return target[func]( this[0]);
		switch( this.length){
		case 0:
			return target[func]();
		case 1:
			return target[func]( this[0]);
		case 2:
			return target[func]( this[0], this[1]);
		case 3:
			return target[func]( this[0], this[1],  this[2]);
		case 4:
			return target[func]( this[0], this[1],  this[2], this[3]);
		case 5:
			return target[func]( this[0], this[1],  this[2], this[3], this[4]);
		case 6:
			return target[func]( this[0], this[1],  this[2], this[3], this[4], this[5]);
		case 7:
			return target[func]( this[0], this[1],  this[2], this[3], this[4], this[5], this[6]);
		case 8:
			return target[func]( this[0], this[1],  this[2], this[3], this[4], this[5], this[6],  this[7]);
		case 9:
			return target[func]( this[0], this[1],  this[2], this[3], this[4], this[5], this[6],  this[7], this[8]);
		case 10:
			return target[func]( this[0], this[1],  this[2], this[3], this[4], this[5], this[6],  this[7], this[8], this[9]);
		case 11:
			return target[func]( this[0], this[1],  this[2], this[3], this[4], this[5], this[6],  this[7], this[8], this[9], this[10]);
		case 12:
			return target[func]( this[0], this[1],  this[2], this[3], this[4], this[5], this[6],  this[7], this[8], this[9], this[10], this[11]);
		case 13:
			return target[func]( this[0], this[1],  this[2], this[3], this[4], this[5], this[6],  this[7], this[8], this[9], this[10], this[11], this[12]);
		case 14:
			return target[func]( this[0], this[1],  this[2], this[3], this[4], this[5], this[6],  this[7], this[8], this[9], this[10], this[11], this[12], this[13]);
		case 15:
			return target[func]( this[0], this[1],  this[2], this[3], this[4], this[5], this[6],  this[7], this[8], this[9], this[10], this[11], this[12], this[13], this[14]);
		case 16:
			return target[func]( this[0], this[1],  this[2], this[3], this[4], this[5], this[6],  this[7], this[8], this[9], this[10], this[11], this[12], this[13], this[14], this[15]);
		default:
			return target[func]( this);
		}
	}

};

JSAX.object.attachTo( JSAX.proto.Array, Array.prototype);
Array.prototype.JSAX = JSAX.proto.Array;










// The shift() and unshift() methods.
// taken from javascriptkit.com

if(!Array.prototype.shift) { // if this method does not exist..

	Array.prototype.shift = function(){
		firstElement = this[0];
		this.reverse();
		this.length = Math.max(this.length-1,0);
		this.reverse();
		return firstElement;
	}
	
}

if(!Array.prototype.unshift) { // if this method does not exist..
	
	Array.prototype.unshift = function(){
		this.reverse();
		
			for(var i=arguments.length-1;i>=0;i--){
				this[this.length]=arguments[i]
			}
			
		this.reverse();
		return this.length
	}
}